/@ 
   VISION SYSTEMS VScom 200 PCI serial card stub driver
   (c) 1998 C.Schroeter  

   The Vision Systems VScom200 Serial interface is a high speed 
   serial board based on standard 16550A circuits and is capable 
   to drive rates up to 921600 baud.

   This driver module is installed using insmod on boot time or 
   kerneld and recognizes the VScom 200 in the PCI area. 
   Both 16550A then are registered with the linux system.

   Be careful using the base_baud and custom_divisor options with 
   the setserial utility, since the driver may not work with the 
   wrong settings.  
   
@/
module "VScom200" : 

driver "VScom200" {

	header "common" /{
#		include <linux/pci.h>
#		include <linux/bios32.h>
#		include <linux/tty.h>
#		include <linux/serial.h>

#		define MAX_LINES 4
	}/

	file "common" /{
		int line_index=0;
		int line[MAX_LINES];
        }/

	core {  
		init /{ 
		       unsigned char pci_bus, pci_device_fn, pci_latency, pci_irq;
	               unsigned int pci_ioaddr0,pci_ioaddr1,pci_ioaddr2,pci_ioaddr3,pci_ioaddr4,pci_ioaddr5;
	               unsigned short pci_command, vendor_id, device_id;

		       extern int line[],line_index;

		       static int pci_index = 0;

                       if (!pcibios_present()) return(-ENODEV);

	               for (; pci_index < 0xff; pci_index++) {
		         if (pcibios_find_class(PCI_CLASS_COMMUNICATION_SERIAL << 8, 
                                               pci_index, &pci_bus, &pci_device_fn) 
                            != PCIBIOS_SUCCESSFUL)
				break;

		         /* get vendor id */
		         pcibios_read_config_word(pci_bus, pci_device_fn, PCI_VENDOR_ID,
								 &vendor_id);
		         /* get IRQ */
		         pcibios_read_config_byte(pci_bus, pci_device_fn, PCI_INTERRUPT_LINE,
								 &pci_irq);

		         /* get device id */
		         pcibios_read_config_word(pci_bus, pci_device_fn, PCI_DEVICE_ID,
								 &device_id);

  		         /* get IO address */
		         pcibios_read_config_dword(pci_bus, pci_device_fn, PCI_BASE_ADDRESS_2,
								  &pci_ioaddr2);
  		         /* get IO address */
		         pcibios_read_config_dword(pci_bus, pci_device_fn, PCI_BASE_ADDRESS_3,
								  &pci_ioaddr3);


		         /* Remove I/O space marker in bit 0. */
		         pci_ioaddr0 &= ~3;
		         pci_ioaddr1 &= ~3;
		         pci_ioaddr2 &= ~3;
		         pci_ioaddr3 &= ~3;
		         pci_ioaddr4 &= ~3;
		         pci_ioaddr5 &= ~3;

		         if (vendor_id != 0x10b5 && device_id != 0x1103 ) {
				printk("VISION SYSTEMS VScom200 Serial PCI not found");
				continue;
			 } else {
			   struct serial_struct serial0;
			   struct serial_struct serial1;


                           printk("VScom200 PCI serial card driver");
			   printk("\n     (c) 1998 C.Schroeter (clausi@chemie.fu-berlin.de) \n");
//			   printk("\n     irq=%d", pci_irq );
//			   printk("\n     base0=0x%4x", pci_ioaddr0 );
//			   printk("\n     base1=0x%4x", pci_ioaddr1 );
//			   printk("\n     base2=0x%4x", pci_ioaddr2 );
//			   printk("\n     base3=0x%4x\n", pci_ioaddr3 );

			   serial0.type  = PORT_UNKNOWN;
			   serial0.port  = pci_ioaddr2;
                           serial0.irq   = pci_irq;
			   serial0.baud_base = 921600;
			   serial0.custom_divisor = 24;
                           serial0.flags = ASYNC_SPD_CUST | ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST | ASYNC_SHARE_IRQ;

			   line[line_index++] = register_serial(&serial0);
                           if (line < 0) {
	                   printk(KERN_NOTICE "serial_cs: register_serial() at 0x%04x, "
	                            "irq %d failed\n", serial0.port, serial0.irq);
	                            return -1;
                           }

			   serial1.type = PORT_UNKNOWN;
                           serial1.port = pci_ioaddr3;
                           serial1.irq  = pci_irq;
			   serial0.baud_base = 921600;
			   serial0.custom_divisor = 24;
                           serial1.flags = ASYNC_SPD_CUST | ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST | ASYNC_SHARE_IRQ;

			   line[line_index++] = register_serial(&serial1);
                           if (line < 0) {
	                   printk(KERN_NOTICE "serial_cs: register_serial() at 0x%04x, "
	                            "irq %d failed\n", serial1.port, serial1.irq);
	                            return -1;
                           }


			}

	                }
		}/

		cleanup /{
			extern int line[],line_index;
			int i;
			for( i=0; i<line_index; i++ )
				unregister_serial(line[i]);
		}/
	}

}
